package com.caimei.service.order.impl;

import com.alibaba.fastjson.JSONArray;
import com.caimei.entity.*;
import com.caimei.mapper.order.OrderMapper;
import com.caimei.mapper.user.LoginMapper;
import com.caimei.mapper.user.PersonalMapper;
import com.caimei.service.order.OrderService;
import com.caimei.utils.AppUtils;
import com.caimei.utils.NoUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Service
public class OrderServiceImpl implements OrderService {
    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private PersonalMapper personalMapper;
    @Autowired
    private LoginMapper loginMapper;
    @Value("${miniprogram.domain}")
    private String domain;

    @Override
    public Map<String, Object> confirmOrder(Integer userId, String classifyIDS, Integer organizeID, String productIDs, Integer count) {
        Map<String, Object> map = new HashMap();
        //商品信息
        List<String> list = new ArrayList();
        if (productIDs.contains(",")) {
            String[] split = productIDs.split(",");
            list = Arrays.asList(split);
        } else {
            list.add(productIDs);
        }
        List<Shop> shopList = orderMapper.findShop(list);
        for (Shop shop : shopList) {
            List<CmOrganizeProducts> productsList = new ArrayList();
            Double totalPrice = 0d;
            if (count == null) {
                //购物车购买
                Map<String, Object> params = new HashMap();
                params.put("productIDs", list);
                params.put("shopID", shop.getShopID());
                params.put("userId", userId);
                productsList = orderMapper.findAllCommodity(params);
                for (CmOrganizeProducts products : productsList) {
                    products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                    totalPrice += products.getRetailPrice() * products.getProductCount();
                }
            } else {
                //立即购买
                CmOrganizeProducts products = orderMapper.findCommodity(productIDs, shop.getShopID());
                products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                products.setProductCount(count);
                totalPrice = products.getRetailPrice() * products.getProductCount();
                productsList.add(products);
            }
            shop.setLogo(AppUtils.getImageURL("shopLogo", shop.getLogo(), 0, domain));
            shop.setTotalPrice(totalPrice);
            shop.setProductsList(productsList);
        }
        //发票信息
        UserInvoiceInfo invoice = personalMapper.findInvoice(userId);
        //运费
        FreePostageCard freePostageCard = orderMapper.queryFree(userId);
        String freight = "";
        if (freePostageCard != null && classifyIDS.contains("3")) {
            freight = "仪器到付，产品包邮";
        } else if (freePostageCard != null) {
            freight = "包邮";
        } else {
            freight = "到付";
        }
        //客服热线号码
        String cellPhone = loginMapper.cellPhone(organizeID);
        //可用余额
        Double userMoney = personalMapper.ableUserMoney(userId);
        if (userMoney == null) userMoney = 0d;
        map.put("shopList", shopList);
        map.put("invoice", invoice);
        map.put("freight", freight);
        map.put("cellPhone", cellPhone);
        map.put("userMoney", userMoney);
        return map;
    }

    @Override
    public String shareOrder(Integer orderID) {
        OrderShareCode orderShareCode = orderMapper.queryCode(orderID);
        //生成分享码
        Integer shareCode = (int) ((Math.random() * 9 + 1) * 1000);
        //过期时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DATE, 1);
        if (orderShareCode != null && orderShareCode.getExpiredTime().compareTo(new Date()) == 1) {
            return orderShareCode.getShareCode();
        } else if (orderShareCode != null) {
            orderShareCode.setShareCode(String.valueOf(shareCode));
            orderShareCode.setExpiredTime(calendar.getTime());
            orderShareCode.setDelFlag("0");
            orderMapper.updateCode(orderShareCode);
            return String.valueOf(shareCode);
        } else {
            OrderShareCode code = new OrderShareCode();
            code.setOrderID(orderID);
            code.setShareCode(String.valueOf(shareCode));
            code.setExpiredTime(calendar.getTime());
            code.setAddTime(new Date());
            code.setDelFlag("0");
            orderMapper.insertCode(code);
            return String.valueOf(shareCode);
        }
    }

    @Override
    public Map<String, Object> particulars(Integer orderID) {
        Map<String, Object> map = new HashMap();
        //收货地址
        BpOrderUserInfo userInfo = orderMapper.findUserInfo(orderID);
        //订单信息,运费
        CmOrder order = orderMapper.findOrder(orderID);
        order.setOrderMark("#" + order.getOrderID() + "#");
        //商品信息
        String[] ShopOrderIDs = {};
        if (order.getShopOrderIDs().contains(",")) {
            ShopOrderIDs = order.getShopOrderIDs().split(",");
        } else {
            ShopOrderIDs = new String[]{order.getShopOrderIDs()};
        }
        List<CmShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
        for (CmShopOrder shopOrder : shopOrderList) {
            List<CmOrderProduct> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderID());
            for (CmOrderProduct orderProduct : orderProductList) {
                orderProduct.setProductImage(AppUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
            }
            shopOrder.setCmOrderProducts(orderProductList);
            shopOrder.setShopLogo(AppUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
        }
        //发票信息
        UserInvoiceInfo invoice = personalMapper.findInvoice(order.getUserID());
        //支付记录
        List<CmDiscernReceipt> discernReceiptList = orderMapper.findDiscernReceipt(orderID);
        //退款记录
        List<CmReturnedPurchase> returnedPurchaseList = orderMapper.findReturnedPurchase(orderID);
        map.put("order", order);
        map.put("userInfo", userInfo);
        map.put("shopOrderList", shopOrderList);
        map.put("invoice", invoice);
        map.put("discernReceiptList", discernReceiptList);
        map.put("returnedPurchaseList", returnedPurchaseList);
        return map;
    }

    @Override
    public WxJsonModel saveOrderInfo(WxJsonModel wxJsonModel,Integer userId, Integer organizeID, String cartType, Integer addressID, Object orderInfo, String balanceDeductionFlag, Double orderShouldPayFee,Integer cmOperationID) {
        //订单信息转化
        List<Map<String,Object>> orderInfoList = (List<Map<String, Object>>) orderInfo;
        if(null != orderInfoList){
            Integer num = 0;//统计购买所有商品的数量
            double productTotalFee = 0d;//商品总金额 （商品单价乘以数量，再加上税费[默认0]）
            double orderTotalFee = 0d;//小计金额 （商品折后单价乘以数量，再加上税费[默认0]）
            double payTotalFee = 0d;//订单总额（小计金额减去经理折扣后，再加上运费[默认0])
            double payableAmount = 0d;//真实支付金额（订单总额减去抵扣的账户余额）
            for(Map<String,Object> shopOrderInfo : orderInfoList){//遍历订单信息
                String shopId = (String)shopOrderInfo.get("shopId");
                String note = (String)shopOrderInfo.get("note");
                Object productInfo = shopOrderInfo.get("productInfo");//一个子订单对应的商品信息
                if(StringUtils.isEmpty(shopId)){
                    return wxJsonModel.error("1001","供应商数据异常" );
                }
                if(null == productInfo){
                    return wxJsonModel.error("1001","订单商品数据异常" );
                }
                //获取子订单商品信息
                List<Map<String,Object>> productInfoList = (List<Map<String, Object>>) productInfo;
                if(!CollectionUtils.isEmpty(productInfoList)){
                    for(Map<String,Object> product : productInfoList){//遍历所有商品
                        Integer productId = (Integer) product.get("productId");
                        Integer productNum = (Integer) product.get("productNum");
                    }
                }
            }

            //保存主订单
            CmOrder cmOrder = new CmOrder();
            cmOrder.setOrderNo(NoUtils.getCmOrderNo("X"));
            cmOrder.setOrganizeID(organizeID);
            cmOrder.setUserID(userId);
            cmOrder.setBuyUserID(cmOperationID);//操作者的ID(下单人)
            cmOrder.setOrderSubmitType(2);// 2:员工帮会所下单
            cmOrder.setOrderType(null);//小程序
            cmOrder.setHasActProduct("0");
            cmOrder.setSendOutStatus("1");//待發貨
            cmOrder.setProductTotalFee(productTotalFee);
            cmOrder.setOrderTotalFee(orderTotalFee);

            cmOrder.setShopOrderIDs("");//暂无类型
            cmOrder.setStatus(0);
            cmOrder.setReceiptStatus("");//（收款买家）收款状态：1待收款、2部分收款、3已收款
            cmOrder.setPayWay("");//(付款供应商)付款状态：1待付款、2部分付款、3已付款
            cmOrder.setPayFlag("");//是否已支付 未支付0 已支付1
            cmOrder.setOnlinePayFlag("0");//是否能走线上支付 0可以 1不可以 只能线下



            //保存子订单

            //保存订单商品


            //获取用户最新余额，判断是否存在

            //提交订单完成后清理购物车(用户ID,商品ID)
            if(StringUtils.equals("1",cartType)){//购物车提交后清理购物车

            }

        }

        return wxJsonModel.success();
    }


    @Override
    public Map<String, Object> logistics(Integer orderID) {
        Map<String, Object> map = new HashMap();
        //物流商品信息
        CmOrder order = orderMapper.findOrder(orderID);
        String[] ShopOrderIDs = {};
        if (order.getShopOrderIDs().contains(",")) {
            ShopOrderIDs = order.getShopOrderIDs().split(",");
        } else {
            ShopOrderIDs = new String[]{order.getShopOrderIDs()};
        }
        List<CmShopOrder> shopOrderList = orderMapper.findAllShopOrder(ShopOrderIDs);
        for (CmShopOrder shopOrder : shopOrderList) {
            List<CmLogisticsRecord> logisticsRecordList = orderMapper.findLogisticsRecord(shopOrder.getShopOrderID());
            for (CmLogisticsRecord logisticsRecord : logisticsRecordList) {
                logisticsRecord.setImage(AppUtils.getImageURL("product", logisticsRecord.getImage(), 0, domain));
            }
            shopOrder.setCmLogisticsRecords(logisticsRecordList);
        }
        //物流信息
        List<LogisticsInfo> logisticsInfoList = orderMapper.findLogisticsInfo(orderID);
        for (LogisticsInfo logisticsInfo : logisticsInfoList) {
            List<Router> routers = JSONArray.parseArray(logisticsInfo.getInfo(), Router.class);
            logisticsInfo.setRouters(routers);
        }
        map.put("logisticsInfoList", logisticsInfoList);
        map.put("shopOrderList", shopOrderList);
        return map;
    }
}
