package com.caimei.service.products.impl;

import com.caimei.entity.CmMallCart;
import com.caimei.entity.CmOrganizeProducts;
import com.caimei.entity.Page;
import com.caimei.entity.Shop;
import com.caimei.mapper.products.HomePageMapper;
import com.caimei.mapper.products.ShoppingMapper;
import com.caimei.service.products.ShoppingService;
import com.caimei.utils.AppUtils;
import com.github.pagehelper.PageHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ShoppingServiceImpl implements ShoppingService {
    protected static final Logger logger = LoggerFactory.getLogger(ShoppingServiceImpl.class);
    @Autowired
    private ShoppingMapper shoppingMapper;
    @Autowired
    private HomePageMapper homePageMapper;
    @Value("${miniprogram.domain}")
    private String domain;

    @Override
    public Map<String, Object> findAllCart(Integer userID, Integer index, Integer pageSize) {
        Map<String, Object> map = new HashMap();
        PageHelper.startPage(index, pageSize);
        List<Shop> shopList = shoppingMapper.findAllShop(userID);
        if (shopList != null && shopList.size() > 0) {
            for (Shop shop : shopList) {
                List<CmOrganizeProducts> productsList = shoppingMapper.findAllProducts(shop.getShopID(), userID);
                shop.setProductsList(productsList);
                Double totalPrice = 0d;
                for (CmOrganizeProducts products : productsList) {
                    products.setMainImage(AppUtils.getImageURL("product", products.getMainImage(), 0, domain));
                    if (products.getValidFlag().equals("1")) {
                        totalPrice += (products.getRetailPrice() * products.getProductCount());
                    }
                }
                shop.setTotalPrice(totalPrice);
            }
        }
        Page<Shop> page = new Page<>(shopList);
        Integer cartQuantity = homePageMapper.cartQuantity(userID);
        if (cartQuantity == null) cartQuantity = 0;
        logger.info("------------ShoppingServiceImpl.findAllCart.cartQuantity:"+cartQuantity);
        map.put("page", page);
        map.put("cartQuantity", cartQuantity);
        return map;
    }

    @Override
    public void updateNumber(CmMallCart cmMallCart) {
        shoppingMapper.updateNumber(cmMallCart);
    }

    @Override
    public void deleteCart(Integer userID, String productIDs) {
        String[] split = {productIDs};
        if (productIDs.contains(",")) {
            split = productIDs.split(",");
        }
        for (String productID : split) {
            if (productID != null && !productID.equals("")) {
                shoppingMapper.deleteCart(userID, Integer.valueOf(productID));
            }
        }
    }
}
